#
# LSST Data Management System
# Copyright 2008-2017 LSST/AURA.
#
# This product includes software developed by the
# LSST Project (http://www.lsst.org/).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the LSST License Statement and
# the GNU General Public License along with this program.  If not,
# see <http://www.lsstcorp.org/LegalNotices/>.
#

from __future__ import absolute_import, division, print_function

__all__ = ["Exposure"]

from future.utils import with_metaclass
import numpy as np

from lsst.utils import TemplateMeta

from .exposure import ExposureI, ExposureF, ExposureD, ExposureU, ExposureL


class Exposure(with_metaclass(TemplateMeta, object)):

    def __reduce__(self):
        from lsst.afw.fits import reduceToFits
        return reduceToFits(self)

    def convertF(self):
        return ExposureF(self, deep=True)

    def convertD(self):
        return ExposureD(self, deep=True)

    @classmethod
    def Factory(cls, *args, **kwargs):
        """Construct a new Exposure of the same dtype.
        """
        return cls(*args, **kwargs)

    def clone(self):
        """Return a deep copy of self"""
        return self.Factory(self, deep=True)

    def __getitem__(self, imageSlice):
        return self.Factory(self.getMaskedImage()[imageSlice])

    def __setitem__(self, imageSlice, rhs):
        self.getMaskedImage[imageSlice] = rhs.getMaskedImage()


Exposure.register(np.int32, ExposureI)
Exposure.register(np.float32, ExposureF)
Exposure.register(np.float64, ExposureD)
Exposure.register(np.uint16, ExposureU)
Exposure.register(np.int64, ExposureL)
Exposure.alias("I", ExposureI)
Exposure.alias("F", ExposureF)
Exposure.alias("D", ExposureD)
Exposure.alias("U", ExposureU)
Exposure.alias("L", ExposureL)
